/*
** File Name : input.cpp
**
**     User Interface Generator Library For TC++
**     ~    ~         ~
**     Copyright (c) 1990,1991 M.Takayanagi (SSC Co.,Ltd.)
**
**			INPUT CLASS
**                      ~~~~~~~~~~~
*/

#include <stdio.h>
#include <stdlib.h>
#include <dir.h>
#include <ctype.h>
#include <dos.h>
#include <string.h>
#include "uiglib.h"

/*
** P͓͍̓
*/
INPUT_CHAR::INPUT_CHAR(int l, int c, unsigned int a, char *b, char *chk, 
                       FENCE fe, unsigned int fa, char* func):
	    INPUT(l, c, a, b, fe, fa, func)
{
	check = chk;
}

int INPUT_CHAR::activate(PANEL& p)
{
	int ret = PANEL_CONTINUE;
	while (1) {
		p.locate(line,column);
		p.color(ATTR_REVERSE|ATTR_YELLOW);
		p.edit(buffer, 1);
		p.locate(line,column);
		p.color(attribute);
		p.puts(buffer);
		if (p.button_exit) {
			if (p.button_exit->active(p)) {
				break;
			}
		}
		if (strchr(check, *buffer)!=NULL) {
			break;
		}
		printf("\007");
	}
	display(p);
	if (func_name) {
		ret = p.function(func_name);
	}
	return(ret);
}


/*
** ͍̓
*/
INPUT_INT::INPUT_INT(int l, int c, unsigned int a, char *b, int min, int max, 
		     FENCE fe, unsigned int fa, char* func) :
	   INPUT(l, c, a, b, fe, fa, func)
{
	minimum = min;
	maximum = max;
}

int INPUT_INT::activate(PANEL& p)
{
	int	value=0;
	int	ret = PANEL_CONTINUE;
	if (sscanf(buffer, "%d", &value)!=1) {
		value = 0;
	}
	while (1) {
		p.locate(line,column);
		p.color(ATTR_REVERSE|ATTR_YELLOW);
		p.edit(buffer, 1);
		p.locate(line,column);
		p.color(attribute);
		p.puts(buffer);
		if (p.button_exit) {
			if (p.button_exit->active(p)) {
				break;
			}
		}
		if (sscanf(buffer, "%d", &value)==1) {
			if (minimum<=value && value<=maximum) break;
		}
		printf("\007");
	}
	char	format[16];
	sprintf(format,"%%%dd", strlen(buffer));
	sprintf(buffer, format, value);
	display(p);
	if (func_name) {
		ret = p.function(func_name);
	}
	return(ret);
}

/*
** SoCg͍̓
*/
INPUT_LONG::INPUT_LONG(int l, int c, unsigned int a, char *b, long min, long max, 
		       FENCE fe, unsigned int fa, char* func) :
	   INPUT(l, c, a, b, fe, fa, func)
{
	minimum = min;
	maximum = max;
}

int INPUT_LONG::activate(PANEL& p)
{
	long	value=0;
	int	ret = PANEL_CONTINUE;
	if (sscanf(buffer, "%ld", &value)!=1) {
		value = 0;
	}
	while (1) {
		p.locate(line,column);
		p.color(ATTR_REVERSE|ATTR_YELLOW);
		p.edit(buffer, 1);
		p.locate(line,column);
		p.color(attribute);
		p.puts(buffer);
		if (p.button_exit) {
			if (p.button_exit->active(p)) {
				break;
			}
		}
		if (sscanf(buffer, "%ld", &value)==1) {
			if (minimum<=value && value<=maximum) break;
		}
		printf("\007");
	}
	char	format[16];
	sprintf(format,"%%%dld", strlen(buffer));
	sprintf(buffer, format, value);
	display(p);
	if (func_name) {
		ret = p.function(func_name);
	}
	return(ret);
}

/*
** ͍̓
*/
INPUT_DOUBLE::INPUT_DOUBLE(int l, int c, unsigned int a, char *b, char *f,
	   double min, double max, FENCE fe, unsigned int fa, char* func):
	   INPUT(l, c, a, b, fe, fa, func)
{
	minimum = min;
	maximum = max;
	format = f;
}

int INPUT_DOUBLE::activate(PANEL& p)
{
	double	value=0.0;
	int	ret = PANEL_CONTINUE;
	if (sscanf(buffer, "%lf", &value)!=1) {
		value = 0.0;
	}
	while (1) {
		p.locate(line,column);
		p.color(ATTR_REVERSE|ATTR_YELLOW);
		p.edit(buffer, 1);
		p.locate(line,column);
		p.color(attribute);
		p.puts(buffer);
		ret = PANEL_CONTINUE;
		if (p.button_exit) {
			if (p.button_exit->active(p)) {
				break;
			}
		}
		if (sscanf(buffer, "%lf", &value)==1) {
			if (minimum<=value && value<=maximum) break;
		}
		printf("\007");
	}
	sprintf(buffer, format, value);
	display(p);
	if (func_name) {
		ret = p.function(func_name);
	}
	return(ret);
}

/*
**  t@C͍̓
*/
INPUT_FNAME::INPUT_FNAME(int l, int c, unsigned int a, char *b, 
			char *w0, char *w1,  char *w2, 
			FENCE fe, unsigned int fa, char* func):
             INPUT(l,c,a,b,fe,fa,func)
{
	files_button = 0;
	wc0 = w0; wc1 = w1; wc2 = w2;
}

/*
**  t@C͍̓
*/
INPUT_FNAME::INPUT_FNAME(int l, int c, unsigned int a, char *b, BUTTON *btn,
			char *w0, char *w1,  char *w2, 
			FENCE fe, unsigned int fa, char* func):
             INPUT(l,c,a,b,fe,fa,func)
{
	files_button = btn;
	wc0 = w0; wc1 = w1; wc2 = w2;
}

int	INPUT_FNAME::activate(PANEL &p)
{
	p.locate(line,column);
	p.color(ATTR_REVERSE|ATTR_YELLOW);
	p.edit(buffer);
	p.locate(line,column);
	p.color(attribute);
	p.puts(buffer);
	if (files_button==0) files_button = p.button_list;
	if (files_button) {
		if (files_button->active(p)) {
			char	*w0,*w1,*w2;
			char	dir[128], buf[128];
			strcpy(dir, ".");
			sscanf(buffer, "%s", buf);
			char *ptr = strrchr(buf, '\\');
			if (ptr) {
				*ptr = '\0';
				strcpy(dir, buf);
			} else {
				ptr = strrchr(buf, ':');
				if (ptr) {
					*(ptr+1) = '\0';
					strcpy(dir, buf);
				}
			}
			if (strchr(buffer, '*') || strchr(buffer, '?')) {
				sscanf(buffer, "%s", buf);
				char *ptr = strrchr(buf, '\\');
				if (ptr == NULL) {
					ptr = strrchr(buf, ':');
				}
				if (ptr) {
					w0 = ptr+1;
				} else {
					w0 = buf;
				}
				w1=0; w2 = 0;
			} else {
				w0 = wc0; w1 = wc1; w2 = wc2;
			}
			FILES_WINDOW	fw(8,3, dir, w0, w1, w2);
			fw.open();
			if (fw.activate()) {
	 			int	len = strlen(buffer);
				char	frm[16];
				sprintf(frm, "%%-%d.%ds", len, len);
				sprintf(buffer, frm, fw.selected_fname());
			}
			fw.close();
		}
	}
	int ret = PANEL_CONTINUE;
	if (func_name) {
		ret = p.function(func_name);
	}
	return(ret);
}

// end of input.cpp
