/*
** File Name : twin.cpp
**
**     User Interface Generator Library For TC++
**     ~    ~         ~
**     Copyright (c) 1990,1991 M.Takayanagi (SSC Co.,Ltd.)
**
**			TEXT_WINDOW CLASS
**                      ~~~~~~~~~~~~~~~~~
*/

#include <stdio.h>
#include <stdlib.h>
#include <mem.h>
#include <string.h>
#include <ctype.h>
#include <jctype.h>
#include <dir.h>
#include <dos.h>
#include "uiglib.h"

static unsigned char* text_vram = 
		(unsigned char*)0xa0000000;	// CODE VRAM ADDRESS
static unsigned char* attr_vram = 
		(unsigned char*)0xa2000000;	// ATTR VRAM ADDRESS
static const int vram_size = (80*25*2);		// VRAM SIZE


/*
** w͈͂̉ʃNA
*/
void TEXT_WINDOW::clrwindow(int l, int c, int ll, int lc)
{
	for (int i=0;i<ll;i++) {
		unsigned int *cptr = (unsigned int *) 
				     &text_vram[160*(l+i-1)+(c-1)*2];
		unsigned int *aptr = (unsigned int *) 
				     &attr_vram[160*(l+i-1)+(c-1)*2];
		if ((c!=1)&&((*cptr)&0x80)) {
			*(cptr-1) = ' ';
			*(aptr-1) = ATTR_WHITE | ATTR_NOSECRET;
		}
		for (int j=0;j<lc;j++) {
			cptr[j] = ' ';
			aptr[j] = ATTR_WHITE | ATTR_NOSECRET;
		}
	}
}

/*
**  ۰ٱ
*/
void TEXT_WINDOW::rollup_window(int l, int c, int ll, int lc)
{
	unsigned int *cptr0, *cptr1, *aptr0, *aptr1;
	for (int i=0;i<ll-1;i++) {
		cptr0 = (unsigned int *)&text_vram[160*(l+i-1)+(c-1)*2];
		aptr0 = (unsigned int *)&attr_vram[160*(l+i-1)+(c-1)*2];
		cptr1 = (unsigned int *)&text_vram[160*(l+i)+(c-1)*2];
		aptr1 = (unsigned int *)&attr_vram[160*(l+i)+(c-1)*2];
		memcpy(cptr0, cptr1, lc*2);
		memcpy(aptr0, aptr1, lc*2);
	}
	for (int j=0;j<lc;j++) {
		cptr1[j] = ' ';
		aptr1[j] = ATTR_WHITE | ATTR_NOSECRET;
	}
}

/*
**  ۰ٱ
*/
void TEXT_WINDOW::rolldown_window(int l, int c, int ll, int lc)
{
	unsigned int *cptr0, *cptr1, *aptr0, *aptr1;
	for (int i=ll-1;i>0;i--) {
		cptr0 = (unsigned int *)&text_vram[160*(l+i-1)+(c-1)*2];
		aptr0 = (unsigned int *)&attr_vram[160*(l+i-1)+(c-1)*2];
		cptr1 = (unsigned int *)&text_vram[160*(l+i-2)+(c-1)*2];
		aptr1 = (unsigned int *)&attr_vram[160*(l+i-2)+(c-1)*2];
		memcpy(cptr0, cptr1, lc*2);
		memcpy(aptr0, aptr1, lc*2);
	}
	for (int j=0;j<lc;j++) {
		cptr1[j] = ' ';
		aptr1[j] = ATTR_WHITE | ATTR_NOSECRET;
	}
}

/*
** eLXgEChẼRXgN^
*/
TEXT_WINDOW::TEXT_WINDOW(int l, int c, int ll, int lc)
{
	tbuf = new unsigned char[vram_size];
	abuf = new unsigned char[vram_size];
	if (tbuf == 0 || abuf == 0) {
		puts("Cannot Allocate Text Window.");
		exit(1);
	}
	line = l; column = c; lines = ll; columns = lc;
	open_flg = 0;
	edit_mode = BSONLY;
}

/*
**  eLXgEChẼfXgN^
*/
TEXT_WINDOW::~TEXT_WINDOW()
{
	if (open_flg) {
		close();
	}
	delete tbuf;
	delete abuf;
}

/*
** eLXgEChẼTCYύX
*/
void	TEXT_WINDOW::resize(int l, int c, int ll, int lc)
{
	line = l; column = c; lines = ll; columns = lc;
	return;
}

/*
** eLXgEChẼI[v
*/
void	TEXT_WINDOW::open()
{
	mouse_cursor_off();
	open_flg = 1;
	memcpy(tbuf, text_vram, vram_size);
	memcpy(abuf, attr_vram, vram_size);
	clrwindow(line, column, lines, columns);
	mouse_cursor_on();
}

/*
** eLXgEChẼN[Y
*/
void	TEXT_WINDOW::close()
{
	mouse_cursor_off();
	memcpy(text_vram, tbuf, vram_size);
	memcpy(attr_vram, abuf, vram_size);
	mouse_cursor_on();
	open_flg = 0;
}

/*
** rLN^Œ``
*/
void	TEXT_WINDOW::put_box(int line, int column,int lines, int columns, 
                    unsigned int attr)
{
	int y = line - 1;
	int x = column - 1;
	unsigned int *cptr = (unsigned int *)&text_vram[y*160+x*2];
	unsigned int *aptr = (unsigned int *)&attr_vram[y*160+x*2];
	cptr[0] = 0x009c; cptr[columns-1] = 0x009d;
	aptr[0] = aptr[columns-1] = attr|ATTR_NOSECRET;
	for (int i=1;i<columns-1;i++) {
		cptr[i] = 0x0095;
		aptr[i] = attr|ATTR_NOSECRET;
	}
	for (i=1;i<lines-1;i++) {
		cptr = (unsigned int *)&text_vram[(y+i)*160+x*2];
		aptr = (unsigned int *)&attr_vram[(y+i)*160+x*2];
		cptr[0] = cptr[columns-1] = 0x0096;
		aptr[0] = aptr[columns-1] = attr|ATTR_NOSECRET;
	}
	cptr = (unsigned int *)&text_vram[(y+i)*160+x*2];
	aptr = (unsigned int *)&attr_vram[(y+i)*160+x*2];
	cptr[0] = 0x009e; cptr[columns-1] = 0x009f;
	aptr[0] = aptr[columns-1] = attr|ATTR_NOSECRET;
	for (i=1;i<columns-1;i++) {
		cptr[i] = 0x0095;
		aptr[i] = attr|ATTR_NOSECRET;
	}
}

/*
** EChEg\B
*/
void	TEXT_WINDOW::frame(unsigned int attr)
{
	mouse_cursor_off();
	put_box(line, column, lines, columns, attr);
	mouse_cursor_on();
}

/*
** rLN^ŃEChEɒ``
*/
void	TEXT_WINDOW::box(int l, int c, int ll, int lc, unsigned int attr)
{
	mouse_cursor_off();
	put_box(line+l-1, column+c-1, ll, lc, attr);
	mouse_cursor_on();
}

/*
** \ʒu̎ws
*/
void	TEXT_WINDOW::locate(int l, int c)
{
	line_ptr = l;
	column_ptr= c;
	EVENT::text_cursor_locate(line+l-1, column+c-1);
}

/*
** \ʒu̓ǂ݂
*/
void	TEXT_WINDOW::where(int& l, int& c)
{
	l = line_ptr;
	c = column_ptr;
}

/*
** EChẼNA
*/
void	TEXT_WINDOW::clear()
{
	mouse_cursor_off();
	clrwindow(line, column, lines, columns);
	mouse_cursor_on();
}

/*
** \Fw
*/
void	TEXT_WINDOW::color(unsigned int attr)
{
	current_attr = attr|ATTR_NOSECRET;
}

/*
** EChEւ̕o
*/
void	TEXT_WINDOW::puts(char* str)
{
	mouse_cursor_off();
	unsigned int	cbuf[128];
	unsigned int	abuf[128];
	int		x,y;
	unsigned char	*cptr, *aptr;
	unsigned int sjis, jis, hi, lo;
	for (int i=0;str[i];i++) {
		if (iskanji(str[i])) {
			sjis = (str[i]<<8)|(str[i+1]&0xff);
			jis = jmstojis(sjis);
			hi  = (jis >> 8) & 0xff;
			lo  = jis & 0xff;
			cbuf[i]   = (hi-0x20) | (lo << 8) ;
			cbuf[i+1] = (hi-0x20) | (lo << 8) | 0x80;
			abuf[i] = abuf[i+1] = current_attr;
			i++;
		} else {
			cbuf[i] = str[i]&0xff;
			abuf[i] = current_attr;
		}
	}
	y = line  + line_ptr - 2;
	x = column + column_ptr - 2;
	cptr = &text_vram[y*160+x*2];
	aptr = &attr_vram[y*160+x*2];
	memcpy(cptr, cbuf, i*2);
	memcpy(aptr, abuf, i*2);
	mouse_cursor_on();
}

/*
** EChEʒuCTCY|[gB
*/
void	TEXT_WINDOW::stat(int& l, int& c, int& ll, int& lc)
{
	l = line; c = column; ll = lines; lc = columns;
}

/*
**  񒆂̊
*/
int TEXT_WINDOW::check_kanji(char* str, int c)
{
	int	t;
	t = 0;
	for (int i=0;i<=c;i++) {
		if (iskanji(str[i])) {
			t = 1;
			i++;
		} else {
			t = 0;
		}
	}
	return(t);
}

/*
** 󔒕̑}
*/
void TEXT_WINDOW::insert_space(char* str, int len, int col)
{
	int	i;

	if (col >= len) return;
	if (check_kanji(str, len-1)) {
		str[len-2] = ' ';
	}
	for (i=len-2;i>=col;i--) {
		str[i+1] = str[i];
	}
	str[col] = ' ';
}


/*
** ̒ǉ
*/
void TEXT_WINDOW::add_char(char* str, int len, int& col)
{
	unsigned int key0, key1;
	key0 = key_code();
	if (iskanji(key0)) {
		event_wait();
		key1 = key_code();
		if (len - col>=2) {
			insert_space(str, len, col);
			insert_space(str, len, col);
			str[col] = key0;
			str[col+1] = key1;
			col += 2;
		}
	} else {
		if (col < len) {
			insert_space(str, len, col);
			str[col] = key0;
			col ++;
		}
	}
	return;
}

/*
** a`bj@ro`bd̏
*/
void TEXT_WINDOW::back_space(char *str, int len, int& col)
{
	if (col > 0) {
		if (check_kanji(str, col-1)) {
			col -= 2;
		} else {
			col --;
		}
		delete_character(str, len, col);
	}
	return;
}

/*
** Ӽ ޮ
*/
void TEXT_WINDOW::delete_character(char* str, int len, int col)
{
	int	i;
	if (col < len) {
		if (check_kanji(str, col)) {
			for (i=col;i<len-2;i++) {
				str[i] = str[i+2];
			}
			str[i] = ' ';
			str[i+1] = ' ';
		} else {
			for (i=col;i<len-1;i++) {
				str[i] = str[i+1];
			}
			str[i] = ' ';
		}
	}
	return;
}

/*
** ҏW
*/
EVENT_TYPE TEXT_WINDOW::edit(char* str, int cflg)
{
	int		col, len;
	EVENT_TYPE	et;
	text_cursor_on();
	int	l,c;
	where(l, c);
	len = strlen(str);
	for (int i=len-1;i>=0;i--) {
		if (str[i]!=' ') {
			break;
		}
	}
	col = i+1;
	EDIT_MODE eflg = edit_mode;
	while (1) {
		locate(l, c); puts(str);
		locate(l, c+col);
		et = event_wait();
		if (et == KEY_PRESSED) {
			unsigned int key0 = key_code();
			if (key0 & 0xff00) break;
			if (key0 >= ' ' && key0 != KEY_DEL) {
				if (cflg) {
					col = 0;
					memset(str, ' ', len);
					cflg = 0;
				}
				add_char(str, len, col);
			} else {
				if (key0==KEY_BS) {
					back_space(str, len, col);
				} else if (key0==KEY_TAB) {
					eflg = FULLEDIT;
				} else if (eflg==FULLEDIT && key0==KEY_DEL) {
					delete_character(str, len, col);
				} else if (eflg==FULLEDIT && key0==KEY_RIGHT) {
					if (check_kanji(str, col)) {
						col += 2;
					} else {
						col ++;
					}
					if (col>=len) break;
				} else if (eflg==FULLEDIT && key0==KEY_LEFT) {
					if (col>0) {
						if (check_kanji(str, col-1)) {
							col -= 2;
						} else {
							col --;
						}
					} else {
						break;
					}
				} else {
					break;
				}
			}
		} else {
			break;
		}
	}
	text_cursor_off();
	return(et);
}

/*
**  ۰ٱ
*/
void TEXT_WINDOW::rollup(int l, int c, int ll, int lc)
{
	mouse_cursor_off();
	rollup_window(line+l-1, column+c-1, ll, lc);
	mouse_cursor_on();
}

/*
**  ۰ٱ
*/
void TEXT_WINDOW::rolldown(int l, int c, int ll, int lc)
{
	mouse_cursor_off();
	rolldown_window(line+l-1, column+c-1, ll, lc);
	mouse_cursor_on();
}

// end of twin.cpp
